# LearningOnline Network with CAPA
# option list style responses
#
# $Id: optionresponse.pm,v 1.120 2004/09/29 05:47:33 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
package Apache::optionresponse;
use strict;
use Apache::response();
use Apache::lonlocal;

BEGIN {
  &Apache::lonxml::register('Apache::optionresponse',('optionresponse'));
}

sub start_optionresponse {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result='';
  #when in a option response use these
  &Apache::lonxml::register('Apache::optionresponse',('foilgroup','foil','conceptgroup','drawoptionlist'));
  push (@Apache::lonxml::namespace,'optionresponse');
  my $id = &Apache::response::start_response($parstack,$safeeval);
  %Apache::hint::option=();
  undef(%Apache::response::foilnames);
  if ($target eq 'edit') {
    $result.=&Apache::edit::start_table($token).
	"<tr><td>Multiple Option Response Question ".
	&Apache::loncommon::help_open_topic('Option_Response_Problems')."</td><td>Delete:".
	&Apache::edit::deletelist($target,$token)
	."</td><td>&nbsp;".
        &Apache::edit::end_row().
        &Apache::edit::start_spanning_row().
        "\n";
    $result.=&Apache::edit::text_arg('Max Number Of Shown Foils:','max',
				     $token,'4').
        &Apache::edit::select_arg('Randomize Foil Order','randomize',
				  ['yes','no'],$token).
        &Apache::edit::select_arg(&mt('Display of options when printed'),'TeXlayout',
				  [['horizontal',&mt('Normal list')],
				   ['vertical',&mt('Listed in a vertical column')]],$token).
        &Apache::edit::end_row().&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						 $safeeval,'max','randomize',
						 'TeXlayout');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  } elsif ($target eq 'meta') {
    $result=&Apache::response::meta_package_write('optionresponse');
  } elsif ($target eq 'analyze') {
    my $part_id="$Apache::inputtags::part.$id";
    push (@{ $Apache::lonhomework::analyze{"parts"} },$part_id);
  }
  return $result;
}

sub end_optionresponse {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  &Apache::response::end_response;
  pop @Apache::lonxml::namespace;
  &Apache::lonxml::deregister('Apache::optionresponse',('foilgroup','foil','conceptgroup','drawoptionlist'));
  my $result;
  if ($target eq 'edit') { $result=&Apache::edit::end_table(); }
  undef(%Apache::response::foilnames);
  return $result;
}

%Apache::response::foilgroup=();
sub start_foilgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;

  my $result='';
  %Apache::response::foilgroup=();
  $Apache::optionresponse::conceptgroup=0;
  &Apache::response::pushrandomnumber();
  if ($target eq 'edit') {
    my $optionlist="<option></option>\n";
    my $option;
    my @opt;
    eval '@opt ='. &Apache::lonxml::get_param('options',$parstack,$safeeval);
    my $count=1;
    foreach $option (@opt) {
      $optionlist.="<option value=\"$count\">$option</option>\n";
      $count++;
    }
    my $insertlist=&Apache::edit::insertlist($target,$token);
    $result.=&Apache::edit::start_table($token);
    $result.= (<<ENDTABLE);
      <tr><td>Select Options</td>
        <td>
	  Add new Option: <input type="text" name="$Apache::lonxml::curdepth.options" />
        </td>
        <td>Delete an Option:
	  <select name="$Apache::lonxml::curdepth.deleteopt">$optionlist</select>
ENDTABLE
    $result.= &Apache::edit::end_row();
    $result.= &Apache::edit::start_spanning_row();
    $result.= $insertlist.'<br />';
  }
  if ($target eq 'modified') {
    my @options;
    my $optchanged=0;
    eval '@options ='.&Apache::lonxml::get_param('options',$parstack,$safeeval);
    if ($ENV{"form.$Apache::lonxml::curdepth.deleteopt"}) {
      my $delopt=$ENV{"form.$Apache::lonxml::curdepth.deleteopt"};
      &Apache::lonxml::debug("Deleting :$delopt:");
      splice(@options,$delopt-1,1);
      $optchanged=1;
    }
    if ($ENV{"form.$Apache::lonxml::curdepth.options"}) {
      my $newopt = $ENV{"form.$Apache::lonxml::curdepth.options"};
      if ($options[0]) {
	push(@options,$newopt);
      } else {
	$options[0]=$newopt;
      }
      $optchanged=1;
    }
    if ($optchanged) {
      $result = "<foilgroup options=\"(";
      foreach my $option (@options) {
	$option=~s/\'/\\\'/g;
	&Apache::lonxml::debug("adding option :$option:");
	$result .="'".$option."',";
      }
      chop $result;
      $result.=')">';
    } # else nothing changed so just use the default mechanism
  }
  if ($target eq 'tex' and $Apache::lonhomework::type ne 'exam') {
      $result .= ' \renewcommand{\labelenumi}{\Alph{enumi}.}
                   \begin{enumerate} ';
  }
  return $result;
}

sub end_foilgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  
  my $result;
  if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' ||
      $target eq 'tex' || $target eq 'analyze') {
    my $tex_option_switch=&Apache::lonxml::get_param('texoptions',$parstack,$safeeval);
    my $name;
    my @opt;
    eval '@opt ='.&Apache::lonxml::get_param('options',$parstack,$safeeval);
    my $TeXlayout=&Apache::lonxml::get_param('TeXlayout',$parstack,$safeeval,
					     -2,0);
    if ($target eq 'tex' && $tex_option_switch eq 'nochoice') {@opt=();}
    &Apache::lonxml::debug("Options are $#opt");
    my $max = &Apache::lonxml::get_param('max',$parstack,$safeeval,'-2');
    my $randomize = &Apache::lonxml::get_param('randomize',$parstack,
					       $safeeval,'-2');
    if ($target eq 'web' || $target eq 'tex') {
      $result.=&displayfoils($target,$max,$randomize,$TeXlayout,@opt);
    } elsif ( $target eq 'answer') {
      $result.=&displayanswers($max,$randomize,@opt);
    } elsif ( $target eq 'analyze') {
	my @shown = &whichfoils($max,$randomize);
	&Apache::response::analyze_store_foilgroup(\@shown,
						  ['text','value','location']);
	my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
	push (@{ $Apache::lonhomework::analyze{"$part_id.options"} },@opt);
    } elsif ( $target eq 'grade') {
      if ( defined $ENV{'form.submitted'}) {
	my @whichopt = &whichfoils($max,$randomize);
	my $temp=1;my $name;
	my %responsehash;
	my %grade;
	my $right=0;
	my $wrong=0;
	my $ignored=0;
	foreach $name (@whichopt) {
	  my $response=&Apache::response::getresponse($temp);
	  if ($ENV{'form.submitted'} eq 'scantron' && $response=~/\S/) {
	      $response = $opt[$response];
	  }
	  if ( $response =~ /[^\s]/) {
	    $responsehash{$name}=$response;
	    my $value=$Apache::response::foilgroup{$name.'.value'};
	    &Apache::lonxml::debug("submitted a $response for $value<br />\n");
	    if ($value eq $response) {
		$grade{$name}='1'; $right++;
	    } else {
		$grade{$name}='0'; $wrong++;
	    }
	  } else {
	    $ignored++;
	  }
	  $temp++;
	}
	my $part=$Apache::inputtags::part;
	my $id = $Apache::inputtags::response['-1'];
	my $responsestr=&Apache::lonnet::hash2str(%responsehash);
	my $gradestr   =&Apache::lonnet::hash2str(%grade);
	my %previous=&Apache::response::check_for_previous($responsestr,
							   $part,$id);
	&Apache::lonxml::debug("Got $right right and $wrong wrong, and $ignored were ignored");
	$Apache::lonhomework::results{"resource.$part.$id.submission"}=
	    $responsestr;
	$Apache::lonhomework::results{"resource.$part.$id.submissiongrading"}=$gradestr;
	
	if (!$Apache::lonhomework::scantronmode) {
	    my $ad;
	    if ($wrong==0 && $ignored==0) {
		$ad='EXACT_ANS';
	    } elsif ($wrong==0 && $right==0) {
		#nothing submitted
	    } else {
		if ($ignored==0) {
		    $ad='INCORRECT';
		} else {
		    $ad='MISSING_ANSWER';
		}
	    }
	    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
	    &Apache::response::handle_previous(\%previous,$ad);
	} else {
	    my $ad;
	    if ($wrong==0 && $right==0) {
		#nothing submitted only assign a score if we 
		#need to override a previous grade
		if (defined($Apache::lonhomework::history{"resource.$part.$id.awarddetail"})) {
		    $ad='ASSIGNED_SCORE';
		}
	    } else {
		$ad='ASSIGNED_SCORE';
	    }
	    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
	    $Apache::lonhomework::results{"resource.$part.$id.awarded"}=
		$right/(scalar(@whichopt));
	    $Apache::lonhomework::results{"resource.$part.$id.numfoils"}=
		scalar(@whichopt);
	}
      }
    }
    &Apache::lonxml::increment_counter(&getfoilcounts($max));
  } elsif ($target eq 'edit') {
    $result.=&Apache::edit::end_table();
  }
  if ($target eq 'tex' and $Apache::lonhomework::type ne 'exam') {
      $result .= '\end{enumerate}';
  }
  &Apache::response::poprandomnumber();
  return $result;
}

sub getfoilcounts {
  my ($max)=@_;
  # +1 since instructors will count from 1
  my $count = $#{ $Apache::response::foilgroup{'names'} }+1;
  if (&Apache::response::showallfoils()) { $max=$count; }
  if ($count>$max) { $count=$max } 
  &Apache::lonxml::debug("Count is $count from $max");
  return $count;
}

sub whichfoils {
    my ($max,$randomize)=@_;
    return &Apache::response::whichorder($max,$randomize,
					 &Apache::response::showallfoils(),
					 \%Apache::response::foilgroup);
}

sub displayanswers {
  my ($max,$randomize,@opt)=@_;
  if (!defined(@{ $Apache::response::foilgroup{'names'} })) {return;}
  my @names = @{ $Apache::response::foilgroup{'names'} };
  my @whichopt = &whichfoils($max,$randomize);
  my $result=&Apache::response::answer_header('optionresponse');
  foreach my $name (@whichopt) {
    $result.=&Apache::response::answer_part('optionresponse',
		     $Apache::response::foilgroup{$name.'.value'})
  }
  $result.=&Apache::response::answer_footer('optionresponse');
  return $result;
}

sub check_for_invalid {
    my ($names,$options) = @_;
    my %bad_names;
    foreach my $name (@{ $names }) {
	my $value=$Apache::response::foilgroup{$name.'.value'};
	my $found=0;
	foreach my $option (@{ $options }) {
	    if ($value eq $option) { $found=1; }
	}
	if (!$found) { $bad_names{$name}=$value; }
    }
    if (%bad_names) {
	my $error=&mt('The question can not be gotten correct, '.
	    'the following foils in the &lt;optionresponse&gt; '.
	    'have invalid correct options').' <br /><tt>'.
	    join('<br />',(map { $_=&mt("[_1] with value [_2]",$_,$bad_names{$_}) } (keys(%bad_names)))).
	    "</tt>";
	&Apache::lonxml::error($error);
    }
}

sub displayfoils {
  my ($target,$max,$randomize,$TeXlayout,@opt)=@_;
  if (!defined(@{ $Apache::response::foilgroup{'names'} })) {return;}
  my @names = @{ $Apache::response::foilgroup{'names'} };
  my @truelist;
  my @falselist;
  my $result;  
  my $name;
  my $displayoptionintex=1;
  my @alphabet = ('A'..'Z');
  my @whichopt = &whichfoils($max,$randomize);
  &check_for_invalid(\@whichopt,\@opt);
  my $part=$Apache::inputtags::part;
  my $id=$Apache::inputtags::response[-1];
  my $break;
  my $solved=$Apache::lonhomework::history{"resource.$part.solved"};
  if ( ($target ne 'tex') &&
       &Apache::response::show_answer() ) {
    my $temp=1;
    foreach $name (@whichopt) {
	my $text=$Apache::response::foilgroup{$name.'.text'};
	my %lastresponse=&Apache::lonnet::str2hash($Apache::lonhomework::history{"resource.$part.$id.submission"});
	my $lastopt=$lastresponse{$name};
	if ($text!~/^\s*$/) { $break='<br />'; }
	$result.=$break;
	if ($target eq 'web') {
	    my $value=$Apache::response::foilgroup{$name.'.value'};
	    if (!($text=~s|<drawoptionlist\s*/>|$value|)) {
		if ($text=~/^\s*$/) {
		    $text=$value.$text;
		} else {
		    $text=$value.': '.$text;
		}
	    } else {
		$text='&#149;'.$text;
	    }
	    $result.=$text."\n";
	}
      if ($Apache::lonhomework::type eq 'exam') {
	  $result.=&webbubbles(\@opt,\@alphabet,$temp,$lastopt);
      }
      $temp++;
    }
  } else {
    my $temp=1;
    my %lastresponse=&Apache::lonnet::str2hash($Apache::lonhomework::history{"resource.$part.$id.submission"});
    my $internal_counter=$Apache::lonxml::counter;
    foreach $name (@whichopt) {
      my $text=$Apache::response::foilgroup{$name.'.text'};
      if ($text!~/^\s*$/) {
	  if ($target eq 'tex') {
	      $break='\vskip 0 mm ';
	  } elsif ($target eq 'web') {
	      $break='<br />';
	  }
      }
      my $lastopt=$lastresponse{$name};
      my $optionlist="<option></option>\n";
      my $option;
      foreach $option (@opt) {
	if ($option eq $lastopt) {
	  $optionlist.="<option selected=\"on\">$option</option>\n";
	} else {
	  $optionlist.="<option>$option</option>\n";
	}
      }
      if ($target ne 'tex') {
	  if ($Apache::lonhomework::type ne 'exam') {
	      $optionlist='<select name="HWVAL_'.
		  $Apache::inputtags::response['-1'].':'.$temp.'">'.
		  $optionlist."</select>\n";
	  } else {
	      $optionlist='<u>'.('&nbsp;'x10).'</u>';
	  }
	  if ($text=~s|<drawoptionlist\s*/>|$optionlist|) {
	      if ($Apache::lonhomework::type ne 'exam') {
		  $text='&#149;'.$text;
	      }
	  } else {
	      if ($Apache::lonhomework::type ne 'exam') {
		  $text=$optionlist.$text;
	      }
	  }
	  $result.=$break.$text."\n";
	  if ($Apache::lonhomework::type eq 'exam') {
	      $result.=&webbubbles(\@opt,\@alphabet,$temp,$lastopt);
	  }
	  $temp++;
      } else {
	  my $texoptionlist='';
	  if ($displayoptionintex &&
	      $Apache::lonhomework::type ne 'exam') {
	      $texoptionlist = &optionlist_correction($TeXlayout,@opt);
	  }
	  if ($text=~/<drawoptionlist\s*\/>/) {
	      $text=~s|<drawoptionlist\s*/>| \\makebox\[0\.3in\]\[b\]\{\\hrulefill\} |;
	  }

	  if ($text=~m/\\item /) {
	      if ($Apache::lonhomework::type eq 'exam') {
	          $text=~s/\\item/\\vskip 2 mm/;
	      }
	      $result.= $texoptionlist.$text;
	  } else {
	      if ($Apache::lonhomework::type eq 'exam') {
		  $result.= $texoptionlist.'  '.$text;
	      } else {
		  if ($text=~/\S/) {$result.= $texoptionlist.'\vspace*{-2 mm}\item '.$text;} else {$result.= $texoptionlist;}
	      }
	  }
	  if ($Apache::lonhomework::type eq 'exam') {
	      $result.='\vskip -1 mm\noindent\begin{enumerate}\item[\textbf{'.
		  $internal_counter.'}.]'.&bubbles(\@alphabet,\@opt).
		  '\end{enumerate} \vskip -8 mm \strut ';
	      $internal_counter++;
	  }
	  $displayoptionintex=0;
      }
    }
  }
  if ($target ne 'tex') {
      return $result.$break;
  } else {
      return $result;
  }
}


sub optionlist_correction {
    my ($TeXlayout,@options) = @_;
    my $texoptionlist='\\item [] Choices: ';
    if ($TeXlayout eq 'vertical') {$texoptionlist='\\item []';}
    if (scalar(@options) > 0) {
	foreach my $option (@options) {
	    $texoptionlist.='{\bf '.
		&Apache::lonxml::latex_special_symbols($option).
		'}';
	    if ($TeXlayout eq 'vertical') {
		$texoptionlist.=' \vskip 0 mm ',
	    } else {
		$texoptionlist.=', ';
	    }
	}
	$texoptionlist=~s/, $//;
	if ($TeXlayout ne 'vertical') {$texoptionlist.='.';}
    } else {
	if ($TeXlayout ne 'vertical') {$texoptionlist='\\item [] \\vskip -5 mm';}
    }
    return $texoptionlist;
}


sub webbubbles {
    my ($ropt,$ralphabet,$temp,$lastopt)=@_;
    my @opt=@$ropt; 
    my @alphabet=@$ralphabet;
    my $result='';
    my $number_of_bubbles = $#opt + 1;
    $result.= '<table border="1"><tr>';
    for (my $ind=0;$ind<$number_of_bubbles;$ind++) {
	my $checked='';
	if ($lastopt eq $opt[$ind]) {
	    $checked=' checked="on" ';
	}
	$result.='<td><input type="radio" name="HWVAL_'.
	    $Apache::inputtags::response['-1'].':'.$temp.
	    '" value="'.$opt[$ind].'" '.$checked.' />';
	if ($alphabet[$ind]) {
	    $result.=$alphabet[$ind].': ';
	}
	$result.=$opt[$ind].'</td>';
    }
    $result.='</tr></table>';
    return $result;
}


sub bubbles {
    my ($ralphabet,$ropt,$response) = @_;
    my @alphabet = @$ralphabet;
    my @opt = @$ropt;
    my ($result,$head,$line) =('','','');
    my $number_of_bubbles = $#opt + 1;
    my $current_length = 0;
    my $textwidth;
    if ($ENV{'form.textwidth'} ne '') {
	$ENV{'form.textwidth'}=~/(\d+)/;
	$textwidth=$1;
    } else {
	$ENV{'textwidth'}=~/(\d*)\.?(\d*)/;
	$textwidth=$1.'.'.$2;
    }
    for (my $ind=0;$ind<=$number_of_bubbles;$ind++) {
	my $leftmargin;
	$opt[$ind]=&Apache::lonxml::latex_special_symbols($opt[$ind]);
	if ($response eq 'rankresponse') {$opt[$ind]='Rank '.$opt[$ind];}
	if ($ind==0) {$leftmargin=6;} else {$leftmargin=10;}
	$current_length += (length($opt[$ind])+length($alphabet[$ind])+4)*2;
	if ($current_length<($textwidth-$leftmargin) and $ind!=$number_of_bubbles) {
	    $line.='\hskip -1 mm {\small \textbf{'.$alphabet[$ind].'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';
	    $head.='lr';
	} else {
	    $line=~s/\&\s*$//;
	    $result.='\vskip -2 mm\noindent\begin{tabular}{'.$head.'}'.$line.'\\\\\end{tabular}\vskip 0 mm';
	    $line = '\hskip -1 mm {\small \textbf{'.$alphabet[$ind].'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';;
	    $head ='lr';
	    $current_length = (length($opt[$ind])+length($alphabet[$ind]))*2;
	}

    }
    return $result;
}


sub start_conceptgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  $Apache::optionresponse::conceptgroup=1;
  %Apache::response::conceptgroup=();
  my $result;
  if ($target eq 'edit') {
    $result.=&Apache::edit::tag_start($target,$token,"Concept Grouped Foils");
    $result.=&Apache::edit::text_arg('Concept:','concept',$token,'50').
        &Apache::edit::end_row().&Apache::edit::start_spanning_row();
  }
  if ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,'concept');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  }
  return $result;
}

sub end_conceptgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  $Apache::optionresponse::conceptgroup=0;
  my $result='';
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' ||
      $target eq 'tex' || $target eq 'analyze') {
    #if not there aren't any foils to display and thus no question
      &Apache::response::pick_foil_for_concept($target,
					       ['value','text','location'],
					       \%Apache::hint::option,
					       $parstack,$safeeval);
  } elsif ($target eq 'edit') {
    $result=&Apache::edit::end_table();
  }
  return $result;
}

sub insert_conceptgroup {
  my $result="\n\t\t<conceptgroup concept=\"\">".&insert_foil()."\n\t\t</conceptgroup>\n";
  return $result;
}

sub start_foil {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result='';
  if ($target eq 'web' || $target eq 'tex' || $target eq 'analyze' ) {
      &Apache::lonxml::startredirection;
      if ($target eq 'analyze') {
	  &Apache::response::check_if_computed($token,$parstack,$safeeval,'value');
      }
  } elsif ($target eq 'edit') {
    $result=&Apache::edit::tag_start($target,$token,"Foil");
    my $level='-2';
    if ($$tagstack['-2'] eq 'conceptgroup') { $level = '-3'; }
    my @opt;
    eval '@opt ='.&Apache::lonxml::get_param('options',$parstack,$safeeval,$level);
    $result.=&Apache::edit::text_arg('Name:','name',$token);
    $result.= &Apache::edit::select_or_text_arg('Correct Option:','value',
					       ['unused',(@opt)],$token,'15');
    my $randomize=&Apache::lonxml::get_param('randomize',$parstack,
					     $safeeval,'-3');
    if ($randomize ne 'no') {
	$result.=&Apache::edit::select_arg('Location:','location',
					   ['random','top','bottom'],$token);
    }
    $result .=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,
						 'value','name','location');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  } 
  return $result;
}

sub end_foil {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $text ='';
  my $result = '';
  if ($target eq 'web' || $target eq 'tex' || $target eq 'analyze') { 
      $text=&Apache::lonxml::endredirection;
      if ($target eq 'tex') {$text=~s/\\strut\s*\\\\\s*\\strut/\\vskip 0 mm/;}
      if (($target eq 'tex') and ($Apache::lonhomework::type ne 'exam') and ($text=~/\S/)) {$text = '\vspace*{-2 mm}\item '.$text;} 
  }
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' 
      || $target eq 'tex' || $target eq 'analyze') {
    my $value = &Apache::lonxml::get_param('value',$parstack,$safeeval);
    if ($value ne 'unused') {
      my $name = &Apache::lonxml::get_param('name',$parstack,$safeeval);
      &Apache::lonxml::debug("Got a name of :$name:");
      if (!$name) { $name=$Apache::lonxml::curdepth; }
      &Apache::lonxml::debug("Using a name of :$name:");
      if (defined($Apache::response::foilnames{$name})) {
	  &Apache::lonxml::error(&mt("Foil name <b><tt>[_1]</tt></b> appears more than once. Foil names need to be unique.",$name));
      }
      $Apache::response::foilnames{$name}++;
      my $location =&Apache::lonxml::get_param('location',$parstack,$safeeval);
      if ( $Apache::optionresponse::conceptgroup
	   && !&Apache::response::showallfoils() ) {
	push @{ $Apache::response::conceptgroup{'names'} }, $name;
	$Apache::response::conceptgroup{"$name.value"} = $value;
	if ($target eq 'tex' and $Apache::lonhomework::type eq 'exam') {
	    $Apache::response::conceptgroup{"$name.text"} = '\vskip 4 mm $\triangleright$ '.$text;
	} else {
	    $Apache::response::conceptgroup{"$name.text"} = $text;
	}
	$Apache::response::conceptgroup{"$name.location"} = $location;
      } else {
	push @{ $Apache::response::foilgroup{'names'} }, $name;
	$Apache::response::foilgroup{"$name.value"} = $value;
	if ($target eq 'tex' and $Apache::lonhomework::type eq 'exam') {
	    $Apache::response::foilgroup{"$name.text"} = '\vskip 5 mm $\triangleright$ '.$text;
	} else {
	    $Apache::response::foilgroup{"$name.text"} = $text;
	}
	$Apache::response::foilgroup{"$name.location"} = $location;
      }
    }
  }
  if ($target eq 'edit') {
    $result.= &Apache::edit::tag_end($target,$token,'');
  }
  return $result;
}

sub start_drawoptionlist {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    if ($target ne 'meta') {
	return $token->[4];
    }
}

sub end_drawoptionlist {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    if ($target ne 'meta') {
	return $token->[2];
    }
}

sub insert_foil {
  return '
<foil name="" value="unused">
<startouttext />
<endouttext />
</foil>';
}
1;
__END__
 
